# -*- coding: utf-8 -*-

"""Beward camera controller."""

#
#  Copyright (c) 2019, Andrey "Limych" Khrolenok <andrey@khrolenok.ru>
#  Creative Commons BY-NC-SA 4.0 International Public License
#  (see LICENSE.md or https://creativecommons.org/licenses/by-nc-sa/4.0/)
#

import logging
from typing import Optional

from requests import ConnectTimeout

from beward.const import ALARM_MOTION
from .core import BewardGeneric

_LOGGER = logging.getLogger(__name__)


class BewardCamera(BewardGeneric):
    """Beward camera controller class."""

    # pylint: disable=R0913
    def __init__(self, host, username, password, rtsp_port=None, stream=0,
                 **kwargs):
        super().__init__(host, username, password, **kwargs)

        self.last_motion_timestamp = None
        self.last_motion_image = None

        self.rtsp_port = rtsp_port
        self.stream = stream

        self._live_image_url = None
        self._rtsp_live_video_url = None

    def obtain_uris(self):
        """Set the URIs for the camera."""

        self._live_image_url = self.get_url(
            'images',
            extra_params={
                'channel': 0,
            },
            # Add authentication data
            username=self.username,
            password=self.password,
        )

        if not self.rtsp_port:
            try:
                info = self.get_info('rtsp')
                self.rtsp_port = info.get('RtspPort', 554)
            except ConnectTimeout:
                self.rtsp_port = 554

        url = 'rtsp://%s:%s@%s:%s/av0_%d' % (
            self.username, self.password,
            self.host, self.rtsp_port,
            self.stream,
        )
        self._rtsp_live_video_url = url

    @property
    def live_image_url(self) -> str:
        """Return URL to get live photo from camera."""
        if not self._live_image_url:
            self.obtain_uris()
        return self._live_image_url

    @property
    def rtsp_live_video_url(self) -> str:
        """Return URL to get live video from camera via RTSP protocol."""
        if not self._live_image_url:
            self.obtain_uris()
        return self._rtsp_live_video_url

    @property
    def live_image(self) -> Optional[bytes]:
        """Return bytes of camera image."""
        res = self.query('images', extra_params={'channel': 0})

        if not res.headers.get('Content-Type') in ('image/jpeg', 'image/png'):
            return None

        return res.content

    def _handle_alarm(self, timestamp, alarm, state):
        """Handle alarms from Beward device."""
        super()._handle_alarm(timestamp, alarm, state)

        if alarm == ALARM_MOTION and state:
            self.last_motion_timestamp = timestamp
            self.last_motion_image = self.live_image



# -*- coding: utf-8 -*-

"""Constants."""

#
#  Copyright (c) 2019, Andrey "Limych" Khrolenok <andrey@khrolenok.ru>
#  Creative Commons BY-NC-SA 4.0 International Public License
#  (see LICENSE.md or https://creativecommons.org/licenses/by-nc-sa/4.0/)
#

TIMEOUT = 3

# Error strings
MSG_GENERIC_FAIL = 'Sorry.. Something went wrong...'

# Alarms
ALARM_ONLINE = 'DeviceOnline'
ALARM_MOTION = 'MotionDetection'
ALARM_SENSOR = 'SensorAlarm'
ALARM_SENSOR_OUT = 'SensorOutAlarm'

# Device types
BEWARD_CAMERA = 'camera'
BEWARD_DOORBELL = 'doorbell'

# Beward models
BEWARD_MODELS = {
    BEWARD_CAMERA: '',
    BEWARD_DOORBELL: 'DS03M DS06M DS06A(P) S06A S06M',
}



# -*- coding: utf-8 -*-

"""Utilites."""

#  Copyright (c) 2019, Andrey "Limych" Khrolenok <andrey@khrolenok.ru>
#  Creative Commons BY-NC-SA 4.0 International Public License
#  (see LICENSE.md or https://creativecommons.org/licenses/by-nc-sa/4.0/)

import re


def normalize_fqdn(hostname) -> str:
    """Normalize full qualified domain name."""
    # Convert unicode hostname to punycode and
    # remove the port number from hostname
    hostname = hostname.encode("idna").decode().split(":")[0]

    # Strip exactly one dot from the right, if present
    if hostname[-1] == ".":
        hostname = hostname[:-1]

    return hostname


def is_valid_fqdn(hostname) -> bool:
    """Validate full qualified domain name."""
    hostname = normalize_fqdn(hostname)
    if not hostname or len(hostname) > 253:
        return False
    dn_seq = hostname.split('.')
    if re.match(r'[0-9]+$', dn_seq[-1]):
        return False
    ldh_re = re.compile(r'^[a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?$', re.IGNORECASE)
    return all(ldh_re.match(x) for x in dn_seq)



# -*- coding: utf-8 -*-

"""Python API for Beward Cameras and Doorbells."""

#
#  Copyright (c) 2019, Andrey "Limych" Khrolenok <andrey@khrolenok.ru>
#  Creative Commons BY-NC-SA 4.0 International Public License
#  (see LICENSE.md or https://creativecommons.org/licenses/by-nc-sa/4.0/)
#

import logging

# Will be parsed by setup.py to determine package metadata
from beward.camera import BewardCamera
from beward.const import BEWARD_CAMERA, BEWARD_DOORBELL
from beward.core import BewardGeneric
from beward.doorbell import BewardDoorbell

__author__ = 'Andrey "Limych" Khrolenok <andrey@khrolenok.ru>'
# Please add the suffix "+" to the version after release, to make it
# possible infer whether in development code from the version string
__version__ = '1.0.5'
__website__ = 'https://github.com/Limych/python-beward'
__license__ = 'Creative Commons BY-NC-SA License'

VERSION = __version__

# You really should not `import *` - it is poor practice
# but if you do, here is what you get:
__all__ = [
    'Beward',
    'BewardGeneric',
    'BewardCamera',
    'BewardDoorbell',
]

# http://docs.python.org/2/howto/logging.html#library-config
# Avoids spurious error messages if no logger is configured by the user
logging.getLogger(__name__).addHandler(logging.NullHandler())

_LOGGER = logging.getLogger(__name__)


# pylint: disable=R0903
class Beward:
    """Beward device factory class."""

    @staticmethod
    def factory(host_ip, username, password, **kwargs):
        """Return correct class for device."""

        bwd = BewardGeneric(host_ip, username, password)
        model = bwd.system_info.get('DeviceModel')
        dev_type = bwd.get_device_type(model)
        inst = None

        if dev_type is None:
            raise ValueError(
                'Unknown device "%s"' % model
            )

        if dev_type == BEWARD_CAMERA:
            inst = BewardCamera(host_ip, username, password, **kwargs)

        if dev_type == BEWARD_DOORBELL:
            inst = BewardDoorbell(host_ip, username, password, **kwargs)

        _LOGGER.debug('Factory create instance of %s', inst.__class__)
        return inst



# -*- coding: utf-8 -*-

"""Beward doorbell controller."""

#
#  Copyright (c) 2019, Andrey "Limych" Khrolenok <andrey@khrolenok.ru>
#  Creative Commons BY-NC-SA 4.0 International Public License
#  (see LICENSE.md or https://creativecommons.org/licenses/by-nc-sa/4.0/)
#

import logging

from beward.const import ALARM_SENSOR
from .camera import BewardCamera

_LOGGER = logging.getLogger(__name__)


class BewardDoorbell(BewardCamera):
    """Beward doorbell controller class."""

    def __init__(self, host, username, password, **kwargs):
        super().__init__(host, username, password, **kwargs)

        self.last_ding_timestamp = None
        self.last_ding_image = None

    def _handle_alarm(self, timestamp, alarm, state):
        """Handle alarms from Beward device."""
        super()._handle_alarm(timestamp, alarm, state)

        if alarm == ALARM_SENSOR and state:
            self.last_ding_timestamp = timestamp
            self.last_ding_image = self.live_image



#!/usr/bin/env python
"""
Beward Cameras and Doorbells API module setup
"""

import io
import re
import sys

from setuptools import setup
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    """ PyTest controller. """

    # Code from here:
    # https://docs.pytest.org/en/latest/goodpractices.html#manual-integration

    # pylint: disable=W0201
    def finalize_options(self):
        TestCommand.finalize_options(self)
        # we don't run integration tests which need an actual Beward device
        self.test_args = ['-m', 'not integration']
        self.test_suite = True

    # pylint: disable=C0415,E0401
    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        import shlex

        errno = pytest.main(shlex.split(self.pytest_args))
        sys.exit(errno)


# pylint: disable=C0103
src = io.open('beward/__init__.py', encoding='utf-8').read()
metadata = dict(re.findall("__([a-z]+)__ = '([^']+)'", src))
docstrings = re.findall('"""(.*?)"""', src, re.MULTILINE | re.DOTALL)

NAME = 'beward'

PACKAGES = (
    'beward',
)

AUTHOR_EMAIL = metadata['author']
VERSION = metadata['version']
WEBSITE = metadata['website']
LICENSE = metadata['license']
DESCRIPTION = docstrings[0]

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: Other/Proprietary License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: Implementation :: CPython',
    'Topic :: Home Automation',
    'Topic :: Security',
    'Topic :: Multimedia :: Video :: Capture',
]

with io.open('README.md', encoding='utf-8') as file:
    LONG_DESCRIPTION = file.read()
    LONG_DESCRIPTION_TYPE = 'text/markdown'

# Extract name and e-mail ("Firstname Lastname <mail@example.org>")
AUTHOR, EMAIL = re.match(r'(.*) <(.*)>', AUTHOR_EMAIL).groups()

REQUIREMENTS = list(open('requirements.txt'))
TEST_REQUIREMENTS = list(open('requirements-tests.txt'))

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    author=AUTHOR,
    author_email=EMAIL,
    license=LICENSE,
    url=WEBSITE,
    packages=PACKAGES,
    install_requires=REQUIREMENTS,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESCRIPTION_TYPE,
    classifiers=CLASSIFIERS,
    cmdclass={'pytest': PyTest},
    tests_require=TEST_REQUIREMENTS,
)
